import React, { PureComponent, Fragment } from 'react';
import { Redirect, navigate } from '@reach/router';
import { Grid, GridCell, GridInner } from '@rmwc/grid';
import { Button, ButtonIcon } from '@rmwc/button';
import { TabBar, Tab } from '@rmwc/tabs';
import { Query, Mutation } from 'react-apollo';
import { capitalize, has, startCase, get, unionBy, reject } from 'lodash';
import { plural } from 'pluralize';
import { CircularProgress } from '@rmwc/circular-progress';
import styled from 'styled-components';
import { Fab } from '@rmwc/fab';
import { Formik, Form, Field } from 'formik';
import { Helmet } from 'react-helmet';
import * as yup from 'yup';

import { remote } from '../../graphs';
import {
  FormikSelect,
  FormikCheckbox,
  FormikTextField,
  FormikReferenceField,
  FormikDateField,
  FormikImageField,
  FormikListField,
} from '../ui/forms';
import {
  filterVariables,
  getTypeName,
  isSubObject,
} from '../../providers/RemoteGraphProvider';
import { Editor } from '../ui/Editor';

const FabActions = styled('div')`
  position: fixed;
  right: 2rem;
  bottom: 2rem;
  z-index: 1002;
  button:not(:last-child) {
    margin-right: 1rem;
  }
`;
const Actions = styled(GridCell)`
  background-color: #f5f5f5;
  padding: 1rem;
  button:not(:last-child) {
    margin-right: 1rem;
  }
`;
const DangerButton = styled(Button)`
  --mdc-theme-primary: ${({ theme }) => theme.rmwc.error};
`;

const getInitialValues = (item, modelConfig) => {
  return modelConfig.editFields.reduce((acc, field) => {
    return { ...acc, [field.name]: item[field.name] };
  }, {});
};

class ResourceEdit extends PureComponent {
  state = { activeTab: 0 };
  render() {
    const { activeTab } = this.state;
    const { resource: resourceParam, id: resourceId } = this.props;
    const canBeDeleted = has(
      remote.mutation,
      `delete${capitalize(resourceParam)}`
    );
    const isNew = resourceId === 'new';
    return (
      <Query query={remote.query.modelConfigsConnection}>
        {({ data: { modelConfigsConnection } }) => (
          <Query
            fetchPolicy="cache-and-network"
            query={remote.query[resourceParam]}
            variables={{ where: { id: resourceId } }}>
            {({ loading, data, client }) => {
              if (loading) return null;
              if (!isNew && !data[resourceParam]) {
                return (
                  <Redirect to={`/list/${plural(resourceParam)}`} noThrow />
                );
              }
              // Format fields
              let item = isNew ? {} : data[resourceParam];
              const config = modelConfigsConnection.edges
                .map(e => e.node)
                .find(node => node.type === capitalize(resourceParam));
              // Create validation schema
              const schemaType = remote.schema.types.find(
                ({ name }) => name === config.type
              );
              const schemaFields = schemaType.fields;
              const schemaInputFields = remote.schema.types.find(
                type =>
                  type.name ===
                  `${capitalize(config.type)}${
                    isNew ? 'CreateInput' : 'UpdateInput'
                  }`
              ).inputFields;
              const allSchemaFields = unionBy(
                reject(schemaInputFields, isSubObject),
                schemaFields,
                'name'
              );
              const initialValues = getInitialValues(item, config);
              const title =
                initialValues.name ||
                initialValues.title ||
                capitalize(resourceId);
              const ResourceEditSchema = schemaInputFields.reduce(
                (acc, inputField) => {
                  const typeName = getTypeName(inputField.type);
                  const typeKind = inputField.type.kind;
                  const fieldConfig = config.editFields.find(
                    ({ name }) => name === inputField.name
                  );
                  if (!fieldConfig) return acc;
                  let schema = yup;
                  if (
                    ['String', 'ID'].includes(typeName) ||
                    ['ENUM'].includes(typeKind)
                  ) {
                    schema = schema.string('String');
                  } else if (['DateTime'].includes(typeName)) {
                    schema = schema.date('Please enter a valid date');
                  } else {
                    return acc;
                  }
                  if (['Email'].includes(fieldConfig.type)) {
                    schema = schema.email('Please enter a valid email');
                  }
                  if (typeKind === 'NON_NULL') {
                    schema = schema.required('Required');
                  } else {
                    schema = schema.nullable();
                  }

                  acc[inputField.name] = schema;

                  if (['Password'].includes(fieldConfig.type)) {
                    schema = schema.oneOf(
                      [yup.ref(inputField.name)],
                      'Passwords must match'
                    );
                    acc[`${inputField.name}Confirm`] = schema;
                  }

                  return acc;
                },
                {}
              );
              return (
                <Grid>
                  <Helmet
                    title={`Edit ${capitalize(resourceParam)}: ${title}`}
                  />
                  <GridCell span={12}>
                    <TabBar
                      activeTabIndex={activeTab}
                      onActivate={evt =>
                        this.setState({ activeTab: evt.detail.index })
                      }>
                      <Tab icon="edit">Edit</Tab>
                      {/* <Tab icon="pageview" disabled>
                        Preview
                      </Tab> */}
                    </TabBar>
                  </GridCell>

                  <Formik
                    initialValues={initialValues}
                    validationSchema={yup.object().shape(ResourceEditSchema)}
                    onSubmit={async (
                      values,
                      { resetForm, setSubmitting, setFieldError }
                    ) => {
                      const name = isNew
                        ? `create${capitalize(resourceParam)}`
                        : `update${capitalize(resourceParam)}`;
                      // Call remote mutation
                      try {
                        const { data } = await client.mutate({
                          mutation: remote.mutation[name],
                          variables: {
                            where: values.id ? { id: values.id } : undefined,
                            data: filterVariables(name, values),
                          },
                        });
                        if (isNew) {
                          return navigate(
                            `/edit/${resourceParam}/${data[name].id}`
                          );
                        }
                        // Reset form values with updated data
                        resetForm(getInitialValues(data[name], config));
                      } catch ({ graphQLErrors }) {
                        setSubmitting(false);
                        if (graphQLErrors) {
                          graphQLErrors.forEach(error => {
                            if (error.data) {
                              return setFieldError(
                                error.data.field,
                                error.message
                              );
                            }
                            window.alert(error.message);
                          });
                        }
                      }
                    }}>
                    {({
                      values,
                      isSubmitting,
                      dirty,
                      resetForm,
                      submitForm,
                      setSubmitting,
                    }) => {
                      return (
                        <GridCell span={12}>
                          <Form
                            onKeyDown={evt => {
                              // Handle form submission via keypress
                              const { key, metaKey, ctrlKey } = evt;
                              if (key === 'Enter') {
                                evt.preventDefault();
                              } else if (
                                (key === 's' && metaKey) ||
                                (key === 's' && ctrlKey)
                              ) {
                                evt.preventDefault();
                                if (!isSubmitting && dirty) {
                                  submitForm();
                                }
                              }
                            }}>
                            <GridInner>
                              {config.editFields.map((fieldConfig, idx) => {
                                const schemaField = allSchemaFields.find(
                                  ({ name }) => name === fieldConfig.name
                                );
                                const schemaType = remote.schema.types.find(
                                  ({ name }) => name === fieldConfig.type
                                );
                                const typeKind = schemaType.kind;
                                const typeName = fieldConfig.type;
                                const fieldName = fieldConfig.name;

                                let disabled =
                                  schemaField.__typename !== '__InputValue';

                                if (
                                  (isNew && disabled) ||
                                  !fieldConfig.enabled
                                ) {
                                  return null;
                                }

                                if (
                                  typeName === 'Json' &&
                                  values[fieldName].hasOwnProperty('blocks') &&
                                  values[fieldName].hasOwnProperty('entityMap')
                                ) {
                                  return (
                                    <GridCell span={12} key={fieldName}>
                                      <Field
                                        name={fieldName}
                                        label={startCase(fieldName)}
                                        component={Editor}
                                        config={fieldConfig}
                                      />
                                    </GridCell>
                                  );
                                } else if (typeName === 'Json') {
                                  return (
                                    <GridCell span={12} key={fieldName}>
                                      <Button disabled>
                                        <ButtonIcon icon="storage" />
                                        {startCase(fieldName)} (JSON)
                                      </Button>
                                    </GridCell>
                                  );
                                }
                                if (typeKind === 'ENUM') {
                                  return (
                                    <GridCell span={12} key={fieldName}>
                                      <Field
                                        component={FormikSelect}
                                        name={fieldName}
                                        disabled={disabled}
                                        label={startCase(fieldName)}
                                      />
                                    </GridCell>
                                  );
                                }
                                if (typeName === 'DateTime') {
                                  return (
                                    <GridCell span={12} key={fieldName}>
                                      <Field
                                        component={FormikDateField}
                                        name={fieldName}
                                        disabled={disabled}
                                        label={startCase(fieldName)}
                                      />
                                    </GridCell>
                                  );
                                }
                                if (schemaField.type.kind === 'LIST') {
                                  return (
                                    <GridCell span={12} key={fieldName}>
                                      <Field
                                        component={FormikListField}
                                        name={fieldName}
                                        disabled={disabled}
                                        label={startCase(fieldName)}
                                      />
                                    </GridCell>
                                  );
                                }
                                if (isSubObject(schemaField)) {
                                  const referenceObject = get(item, fieldName);
                                  const referenceType = getTypeName(
                                    schemaField.type
                                  ).toLowerCase();
                                  const referencePath = isNew
                                    ? null
                                    : `/edit/${referenceType}/${
                                        referenceObject.id
                                      }`;
                                  return (
                                    <GridCell span={12} key={fieldName}>
                                      <Field
                                        component={FormikReferenceField}
                                        name={fieldName}
                                        label={startCase(fieldName)}
                                        referenceType={plural(referenceType)}
                                        referenceLabel={fieldConfig.reference}
                                        referencePath={referencePath}
                                      />
                                    </GridCell>
                                  );
                                }
                                if (typeName === 'Boolean') {
                                  return (
                                    <GridCell span={12} key={fieldName}>
                                      <Field
                                        component={FormikCheckbox}
                                        name={fieldName}
                                        disabled={disabled}
                                        label={startCase(fieldName)}
                                      />
                                    </GridCell>
                                  );
                                }
                                if (
                                  typeName === 'String' &&
                                  fieldName === 'src'
                                ) {
                                  return (
                                    <GridCell span={12} key={fieldName}>
                                      <Field
                                        style={{ width: '100%' }}
                                        component={FormikImageField}
                                        name={fieldName}
                                        disabled={disabled}
                                        label={startCase(fieldName)}
                                      />
                                    </GridCell>
                                  );
                                }
                                if (typeName === 'Password') {
                                  return (
                                    <Fragment key={fieldName}>
                                      <GridCell span={12}>
                                        <Field
                                          style={{ width: '100%' }}
                                          type="password"
                                          autoComplete="new-password"
                                          component={FormikTextField}
                                          name={fieldName}
                                          disabled={disabled}
                                          label={
                                            isNew
                                              ? startCase(fieldName)
                                              : 'Enter New Password'
                                          }
                                        />
                                      </GridCell>
                                      <GridCell span={12}>
                                        <Field
                                          style={{ width: '100%' }}
                                          type="password"
                                          disabled={disabled}
                                          autoComplete="new-password"
                                          component={FormikTextField}
                                          name={`${fieldName}Confirm`}
                                          label={
                                            isNew
                                              ? 'Confirm Password'
                                              : 'Confirm New Password'
                                          }
                                        />
                                      </GridCell>
                                    </Fragment>
                                  );
                                }
                                return (
                                  <GridCell span={12} key={fieldName}>
                                    <Field
                                      style={{ width: '100%' }}
                                      disabled={disabled}
                                      component={FormikTextField}
                                      name={fieldName}
                                      label={startCase(fieldName)}
                                    />
                                  </GridCell>
                                );
                              })}
                              <Actions span={12}>
                                <Button
                                  unelevated
                                  type="button"
                                  disabled={isSubmitting || !dirty}
                                  onClick={submitForm}>
                                  {isSubmitting ? (
                                    <span>
                                      <ButtonIcon icon={<CircularProgress />} />{' '}
                                      Saving...
                                    </span>
                                  ) : (
                                    <span>
                                      <ButtonIcon icon="save" /> Save
                                    </span>
                                  )}
                                </Button>
                                <Button
                                  outlined
                                  type="reset"
                                  disabled={isSubmitting || !dirty}
                                  onClick={() => resetForm(initialValues)}>
                                  <ButtonIcon icon="undo" /> Undo
                                </Button>
                                {canBeDeleted && (
                                  <Mutation
                                    mutation={
                                      remote.mutation[
                                        `delete${capitalize(resourceParam)}`
                                      ]
                                    }
                                    onError={error => window.alert(error)}
                                    onCompleted={() => {
                                      navigate(
                                        `/list/${plural(resourceParam)}`
                                      );
                                    }}
                                    variables={{ where: { id: resourceId } }}>
                                    {handleDelete => (
                                      <DangerButton
                                        type="button"
                                        outlined
                                        onClick={() => {
                                          setSubmitting(true);
                                          handleDelete();
                                        }}
                                        disabled={isNew || isSubmitting}>
                                        <ButtonIcon icon="delete" /> Delete
                                      </DangerButton>
                                    )}
                                  </Mutation>
                                )}
                              </Actions>
                              <FabActions>
                                <Fab
                                  icon="save"
                                  label="Save"
                                  exited={isSubmitting || !dirty}
                                  type="submit"
                                />
                                <Fab
                                  icon="add"
                                  type="button"
                                  onClick={() => {
                                    navigate(`/edit/${resourceParam}/new`);
                                    resetForm();
                                  }}
                                />
                              </FabActions>
                            </GridInner>
                          </Form>
                        </GridCell>
                      );
                    }}
                  </Formik>
                  <GridCell style={{ height: 64 }} />
                </Grid>
              );
            }}
          </Query>
        )}
      </Query>
    );
  }
}
export default ResourceEdit;
